<?php

namespace MJ\LoginAsUser\Admin\Controller;

use XF\Mvc\ParameterBag;
use XF\Mvc\FormAction;
use XF\Admin\Controller\AbstractController;

class LoginLog extends AbstractController
{
	public function actionIndex()
	{
		$filters = $this->filterSearchConditions();
		$conditions = [];

		$page = $this->filterPage();
		$perPage = 50;

		if ($this->request->exists('delete_logs'))
		{
			return $this->rerouteController(__CLASS__, 'delete');
		}

		$logRepo = $this->getLoginLogRepo();
		$logFinder = $logRepo->findLoginLogsForList()->limitByPage($page, $perPage);

		if ($username = $filters['username'])
		{
			$user = $this->finder('XF:User')->where('username', $username)->fetchOne();
			if ($user)
			{
				$logFinder->where('user_id', $user->user_id);
				$conditions['username'] = $user->username;
			}
		}

		if ($triggerUsername = $filters['login_username'])
		{
			$user = $this->finder('XF:User')->where('username', $triggerUsername)->fetchOne();
			if ($user)
			{
				$logFinder->where('login_user_id', $user->user_id);
				$conditions['login_username'] = $user->username;
			}
		}

		if ($filters['start'])
		{
			$logFinder->where('log_date', '>', $filters['start']);
			$conditions['start'] = $filters['start'];
		}

		if ($filters['end'])
		{
			$logFinder->where('log_date', '<', $filters['end']);
			$conditions['end'] = $filters['end'];
		}

		if ($conditions && $this->isPost())
		{
			return $this->redirect($this->buildLink('mjlau-login', null, $conditions), '');
		}
		$logFinder->with(['User', 'LoginUser']);
		$total = $logFinder->total();
		$this->assertValidPage($page, $perPage, $total, 'mjlau-login');

		$viewParams = [
			'loginLogs' => $logFinder->fetch(),

			'page'         => $page,
			'perPage'      => $perPage,
			'total'        => $total,

			'conditions'   => $conditions,

			'datePresets'  => \XF::language()->getDatePresets()
		];

		return $this->view('MJ\LoginAsUser:LoginLog\Listing', 'mjlau_log_login_list', $viewParams);
	}

	public function actionDelete(ParameterBag $params)
	{
		$entry = $this->assertLoginLogExists($params->log_id);

		if ($this->isPost())
		{
			$entry->delete();

			return $this->redirect($this->buildLink('mjlau-login'));
		}
		else
		{
			$viewParams = [
				'entry' => $entry
			];
			return $this->view('MJ\LoginAsUser:LoginLog\Delete', 'mjlau_log_login_delete', $viewParams);
		}
	}

	public function actionClear()
	{
		$this->setSectionContext('mjlauReset');

		if ($this->isPost())
		{
			$this->repository('MJ\LoginAsUser:LoginLog')->clearLoginLogs();
			return $this->redirect($this->buildLink('mjlau-login'));
		}
		else
		{
			return $this->view('MJ\LoginAsUser:LoginLog\Reset', 'mjlau_log_login_clear');
		}
	}

	protected function filterSearchConditions($removeEmpty = false)
	{
		$conditions = $this->filter([
			'username'       => 'str',
			'login_username' => 'str',
			'start'          => 'datetime',
			'end'            => 'datetime',
		]);
		if($removeEmpty){
			$conditions = array_filter($conditions);
		}
		return $conditions;
	}

	public function actionRefineSearch()
	{
		$conditions = $this->filterSearchConditions();

		$viewParams = [
			'currencies'       => $this->repository('MJ\LoginAsUser:Currency')->findCurrenciesForList()->fetch(),
			'eventDefinitions' => $this->repository('MJ\LoginAsUser:Event')->getEventDefinitionTitlePairs(true),
			'conditions'       => $conditions,
			'datePresets'      => \XF::language()->getDatePresets()
		];
		return $this->view('XF:Template\RefineSearch', 'mjlau_log_refine_search', $viewParams);
	}

	/**
	 * @param string $id
	 * @param array|string|null $with
	 * @param null|string $phraseKey
	 *
	 * @return \MJ\LoginAsUser\Entity\LoginLog
	 */
	protected function assertLoginLogExists($id, $with = null, $phraseKey = null)
	{
		return $this->assertRecordExists('MJ\LoginAsUser:LoginLog', $id, $with, 'mjs_requested_log_not_found');
	}

	/**
	 * @return \MJ\LoginAsUser\Repository\LoginLog
	 */
	protected function getLoginLogRepo()
	{
		return $this->repository('MJ\LoginAsUser:LoginLog');
	}
}
